;; Solar Powered Circuit Checker
;; FW for ATtiny13A
;; 
;; 2018.02.09 Eleken
;; rev_c1815@yahoo.co.jp
;; http://eleken.y-lab.org/

;; Fuse bits: FSTRT = 1, RSTDISBL = 1, CKSEL[2:0] = 100 (All default)
;; Main clock: 1 MHz

;; Icc (Active) = 1 mA
;; Icc (Idle)   = 0.4 mA max (not uA!)
;;				  0.1uA typ when Vdd = 4V
;; Icc (PDN)    = 5 uA max
;; Open circuit of an interval turns it to PDN mode
;; Reccomend to use tiny13A so as for power consumption!

;; ---------------HOW TO ASSEMBLE-------------
;;  (when using avr-gcc)
;;	avr-gcc -mmcu=attiny11 tiny11_signal.S -o main.elf
;;	avr-objcopy -j .text -j .data -O ihex main.elf main.hex
;;	avr-size main.hex

#include <avr/io.h>
;; Note: Address of IO register is defined in avr/io.h but these addresses are at offset
;; of 0x20 so that normal memory access instruction such as st/ld uses this offset address
;; This means that in/out instruction is NOT used with IO address definition in avr/io.h

PC		= .

;; Port address for OUT/IN instruction
d_DDRB	= DDRB - 32
d_PINB = PINB - 32
d_PORTB = PORTB - 32
d_TCCR0 = TCCR0B - 32
d_TIMSK = TIMSK0 - 32
;d_TCCR0A = TCCR0A - 32
;d_TCCR0B = TCCR0B - 32
;d_TIMSK0 = TIMSK0 - 32
d_WDTCR = WDTCR - 32
d_ACSR = ACSR - 32
d_MCUCR = MCUCR - 32
d_GIMSK = GIMSK - 32
d_GIFR = GIFR - 32
d_TCNT0 = TCNT0 - 32

;; Rename Registers
temp1	= 16
temp2	= 17
temp3	= 18
temp4	= 19
tempIR1	= 20 ;; used only at IR handler
tempIR2	= 21 ;; used only at IR handler

timer_flag	= 2	;; timer is used for beep when 1, otherwise timer used for checking short circuit when 0
dsc_count	= 3	;; used as 1/256 divider
sreg_stack	= 4 ;; IRQ in software must pop / push status register

;; DON'T use R0 (reserved for LPM)

.section	.text

;; wait 1 us
y_wait_us:
	ret


;; 136us / 256ms interrupt
.section	.text
.global		TIM0_OVF_vect
TIM0_OVF_vect:
;	ldi		tempIR1, 0x10	;; swap PB4
;	in		tempIR2, d_PORTB
;	eor		tempIR2, tempIR1
;	out		d_PORTB, tempIR2
	sbrc	timer_flag, 0
	rjmp	TIMER0_beep	; skip if timer_flag == 1
TIMER0_check:
	;; check short circuit
	;; power up analog comparator and wait for 1 us
	dec		dsc_count
	sbi		d_PORTB, 2
	ldi		tempIR1, 0x00	;; ACD = 0
	out		d_ACSR, tempIR1
	rcall	y_wait_us
	
	;; check comparator
TIMER0_c_check_comparator:
	in		tempIR1, d_ACSR ;; bit 5 sets when PB0 > PB1 (short circuit)
	sbrc	tempIR1, 5	;; skip if short circuit is not detected
	rjmp	TIMER0_c_short_circuit
TIMER0_c_open_circuit:
	;; since short circuit is not detected, power down analog comparator
	cbi		d_PORTB, 2
	ldi		tempIR1, 0x80	;; ACD = 1
	out		d_ACSR, tempIR1
	reti
TIMER0_c_short_circuit: ;; short circuit
	;; analog comparator is stay powered on
	;; and change timer flag
	inc		timer_flag
	ldi	tempIR1, 0x01		;; TCCR0 ($33) = 0x01 (Divider 1/1)
	out	d_TCCR0, tempIR1
	reti


TIMER0_beep:
	ldi		tempIR1, -(150-6)
	out		d_TCNT0, tempIR1 ;; timer
	
	;; Beep (swap PB3)
	;; interrupt interval is about 125 us, then beep freqeuncy is 4 kHz
	in		tempIR1, d_PORTB
	ldi		tempIR2, 0x08
	eor		tempIR1, tempIR2
	out		d_PORTB, tempIR1
	
TIMER0_b_check_comparator:
	;; check comparator...
	;; (comparator is enabled in interrupt handler when )
	in		tempIR1, d_ACSR
	sbrc	tempIR1, 5	;; skip if short circuit is not detected
	rjmp	TIMER0_b_short_circuit
TIMER0_b_open_circuit:
	;; since short circuit is not detected, change timer flag
	clr		timer_flag
	ldi	tempIR1, 0x05		;; TCCR0 ($33) = 0x05 (Divider 1/1024)
	out	d_TCCR0, tempIR1
	
	sbi		d_PORTB, 3
	cbi		d_PORTB, 2
	ldi		tempIR1, 0x80
	out		d_ACSR, tempIR1
	reti
TIMER0_b_short_circuit: ;; short circuit
	;; analog comparator is stay powered on
	;; no action here
	reti


.global		INT0_vect
INT0_vect:
	reti

.section	.text

.global		main
main:
	;; Initialize registers
	
	ldi		temp1, 0b00011100
	out		d_DDRB, temp1
	ldi	 temp1, 0b00011000
	out		d_PORTB, temp1
	
	ldi		temp1, 0x05		;; TCCR0 ($33) = 0x05 (Divider 1/1024)
	out		d_TCCR0, temp1
	ldi		temp1, 0x02		;; TIMSK ($39) = 0x02
	out		d_TIMSK, temp1
	
	;; Interrupt and sleep mode
	ldi		temp1, 0x22		;; falling edge of INT0 can generates interrupt, sleep mode is Idle
	out		d_MCUCR, temp1
;;	ldi		temp1, 0x40
;;	out		d_GIMSK, temp1
	
	ldi		temp1, 0x80
	out		d_ACSR, temp1
	
	clr		timer_flag
	ldi		temp1, 10
	mov		dsc_count, temp1
	
	;; Init Z regs and other regs
;;	ldi		ZL, 0
;;	ldi		ZH, 5

	
	sei		;; Interrupt enable
	
main_loop:
	sleep
	tst		dsc_count
	brne	main_loop
	rjmp	deep_sleep_check


;; Turn to power-down sleep
;; (modified [18/04/27])
;; This routine is called every 2.5 sec
deep_sleep_check:
	ldi		temp1, 10
	mov		dsc_count, temp1
	
;	ldi		temp1, 0b00011011
;	out		d_PORTB, temp1
	sbis	d_PINB, 1	;; skip if PB1 = H
	rjmp	main_loop
	
deep_sleep:
	ldi		temp1, 0x00		;; stop timer
	out		d_TCCR0, temp1
	
	ldi		temp1, 0x80	;; ACD = 1
	out		d_ACSR, temp1
	
	ldi		temp1, 0x40
	out		d_GIMSK, temp1
	
	ldi		temp1, 0x30		;; Low level of INT0 can generates interrupt, sleep mode is Power-down
	;; Note that pin change interrupt in INT0 couldn't wake up from deep sleep
	out		d_MCUCR, temp1
	
	nop
	nop
	
	sleep
	
	ldi		temp1, 0x00
	out		d_GIMSK, temp1
	
	ldi		temp1, 0x22		;; falling edge of INT0 can generates interrupt, sleep mode is Idle
	out		d_MCUCR, temp1
	
	ldi		tempIR1, 250
	out		d_TCNT0, tempIR1 ;; timer
	
	ldi		temp1, 0x05		;; TCCR0 ($33) = 0x05 (Divider 1/1024)
	out		d_TCCR0, temp1
	
;	ldi		temp1, 0b00011000
;	out		d_PORTB, temp1

	rjmp	main_loop


.end

